@@ -68,6 +68,7 @@ void usage(void)
 	printf("-wh x	height of window\n");
 	printf("-S	prepend merged output with subwindow-number\n");
 	printf("-f	follow the following filename, not the descriptor\n");
+	printf("--follow-all	for all files after this switch; follow the filename instead of the descriptor\n");
 	printf("--retry	keep trying to open the following file if it is inaccessible\n");
 	printf("--retry-all like --retry but for all following files\n");
 	printf("-fr scheme  use the predefined filter from the configfile\n");
@@ -155,27 +156,28 @@ void add_redir_to_file(char mode, char *file, redirect_t **predir, int *n_redire
 void add_redir_to_proc(char mode, char *proc, redirect_t **predir, int *n_redirect)
 {
 	int fds_to_proc[2], fds_from_proc[2];
-	int cur_index = (*n_redirect)++;
 
 	assert(mode == 'G' || mode == 'g');
 
-	*predir = (redirect_t *)myrealloc(*predir, (*n_redirect) * sizeof(redirect_t), __FILE__, __PRETTY_FUNCTION__, __LINE__);
+	*predir = (redirect_t *)myrealloc(*predir, (*n_redirect + 1) * sizeof(redirect_t), __FILE__, __PRETTY_FUNCTION__, __LINE__);
 
-	if ((*predir)[cur_index].type != REDIRECTTO_NONE) error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "One can only set one redirection-type per (sub-)window.\n");
+	if ((*predir)[*n_redirect].type != REDIRECTTO_NONE) error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "One can only set one redirection-type per (sub-)window.\n");
 
 	if (mode == 'g')
-		(*predir)[cur_index].type = REDIRECTTO_PIPE_FILTERED;
+		(*predir)[*n_redirect].type = REDIRECTTO_PIPE_FILTERED;
 	else
-		(*predir)[cur_index].type = REDIRECTTO_PIPE;
+		(*predir)[*n_redirect].type = REDIRECTTO_PIPE;
 
-	(*predir)[cur_index].redirect = mystrdup(proc, __FILE__, __PRETTY_FUNCTION__, __LINE__);
+	(*predir)[*n_redirect].redirect = mystrdup(proc, __FILE__, __PRETTY_FUNCTION__, __LINE__);
 
-	(*predir)[cur_index].pid = exec_with_pipe((*predir)[cur_index].redirect, fds_to_proc, fds_from_proc);
+	(*predir)[*n_redirect].pid = exec_with_pipe((*predir)[*n_redirect].redirect, fds_to_proc, fds_from_proc);
 	myclose(fds_to_proc[0]);
 	myclose(fds_from_proc[0]);
 	myclose(fds_from_proc[1]);
 
-	(*predir)[cur_index].fd = fds_to_proc[1];
+	(*predir)[*n_redirect].fd = fds_to_proc[1];
+
+	(*n_redirect)++;
 }
 
 void add_redir_to_socket(char filtered, char *prio, char *fac, char *address, redirect_t **predir, int *n_redirect)
@@ -185,33 +187,33 @@ void add_redir_to_socket(char filtered, char *prio, char *fac, char *address, re
 	char *colon = strchr(local_address, ':');
 	int prio_nr = -1, fac_nr = -1;
 	int loop;
-	int cur_index = (*n_redirect)++;
+
 	*predir = (redirect_t *)myrealloc(*predir, (*n_redirect) * sizeof(redirect_t), __FILE__, __PRETTY_FUNCTION__, __LINE__);
 
 	assert(filtered == 1 || filtered == 0);
 
 	if (filtered)
-		(*predir)[cur_index].type = REDIRECTTO_SOCKET_FILTERED;
+		(*predir)[*n_redirect].type = REDIRECTTO_SOCKET_FILTERED;
 	else
-		(*predir)[cur_index].type = REDIRECTTO_SOCKET;
+		(*predir)[*n_redirect].type = REDIRECTTO_SOCKET;
 
-	(*predir)[cur_index].redirect = mystrdup(address, __FILE__, __PRETTY_FUNCTION__, __LINE__);
+	(*predir)[*n_redirect].redirect = mystrdup(address, __FILE__, __PRETTY_FUNCTION__, __LINE__);
 
-	(*predir)[cur_index].fd = socket(AF_INET, SOCK_DGRAM, 0);
-	if ((*predir)[cur_index].fd == -1)
+	(*predir)[*n_redirect].fd = socket(AF_INET, SOCK_DGRAM, 0);
+	if ((*predir)[*n_redirect].fd == -1)
 		error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "Cannot create socket for redirecting via syslog protocol.\n");
 	
-	memset(&(*predir)[cur_index].sai, 0x00, sizeof((*predir)[cur_index].sai));
-	(*predir)[cur_index].sai.sin_family = AF_INET;
+	memset(&(*predir)[*n_redirect].sai, 0x00, sizeof((*predir)[*n_redirect].sai));
+	(*predir)[*n_redirect].sai.sin_family = AF_INET;
 	if (colon)
 	{
 		*colon = 0x00;
-		(*predir)[cur_index].sai.sin_port = atoi(colon + 1);
+		(*predir)[*n_redirect].sai.sin_port = atoi(colon + 1);
 	}
 	else
-		(*predir)[cur_index].sai.sin_port = 514;
+		(*predir)[*n_redirect].sai.sin_port = 514;
 	hp = gethostbyname(local_address);
-	memcpy(&(*predir)[cur_index].sai.sin_addr.s_addr, hp -> h_addr, hp -> h_length);
+	memcpy(&(*predir)[*n_redirect].sai.sin_addr.s_addr, hp -> h_addr, hp -> h_length);
 	
 	for(loop=0; loop<8; loop++)
 	{
@@ -235,9 +237,11 @@ void add_redir_to_socket(char filtered, char *prio, char *fac, char *address, re
 	if (fac_nr == -1)
 		error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "Facility '%s' is not known.\n", fac);
 
-	(*predir)[cur_index].prio_fac = (fac_nr * 8) + prio_nr;
+	(*predir)[*n_redirect].prio_fac = (fac_nr * 8) + prio_nr;
 
 	myfree(local_address);
+
+	(*n_redirect)++;
 }
 
 void argv_set_window_widths(char *widths)
@@ -472,6 +476,7 @@ int argv_add_stripper(char *mode, char *pars[], strip_t **pstrip, int *n_strip)
 		else if (mode[2] == 'S')
 			(*pstrip)[*n_strip].type = STRIP_KEEP_SUBSTR;
 		(*pstrip)[*n_strip].regex_str = mystrdup(re, __FILE__, __PRETTY_FUNCTION__, __LINE__);
+		(*pstrip)[*n_strip].del = NULL;
 		compile_re(&(*pstrip)[*n_strip].regex, re);
 		(*n_strip)++;
 	}
@@ -481,6 +486,7 @@ int argv_add_stripper(char *mode, char *pars[], strip_t **pstrip, int *n_strip)
 		(*pstrip)[*n_strip].type = STRIP_TYPE_RANGE;
 		(*pstrip)[*n_strip].start = get_value_arg(mode, pars[n_pars_used++], VAL_ZERO_POSITIVE);
 		(*pstrip)[*n_strip].end = get_value_arg(mode, pars[n_pars_used++], VAL_ZERO_POSITIVE);
+		(*pstrip)[*n_strip].del = NULL;
 		if ((*pstrip)[*n_strip].end <= (*pstrip)[*n_strip].start)
 			error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "'-kr start end': end must be higher then start.\n");
 		(*n_strip)++;
@@ -852,6 +858,11 @@ void do_commandline(int argc, char *argv[])
 		{
 			follow_filename = 1;
 		}
+		else if (strcmp(argv[loop], "--follow-all") == 0)
+		{
+			default_follow_filename = 1;
+			follow_filename = 1;
+		}
 		else if (strcmp(argv[loop], "-w") == 0)
 		{
 			use_colors = 0;
@@ -874,7 +885,7 @@ void do_commandline(int argc, char *argv[])
 		}
 		else if (strcasecmp(argv[loop], "--label") == 0)
 		{
-			label = argv[++loop];
+			label = mystrdup(argv[++loop], __FILE__, __PRETTY_FUNCTION__, __LINE__);
 		}
 		else if (strcasecmp(argv[loop], "-i") == 0 || argv[loop][0] != '-' ||
 				strcasecmp(argv[loop], "-l") == 0 ||
