@@ -8,12 +8,17 @@
 #include <stdlib.h>
 #include <signal.h>
 #include <regex.h>
+#if defined(__GLIBC__)
+#include <execinfo.h>
+#endif
+#include <sys/socket.h>
+#include <netinet/in.h>
 
 #include "mt.h"
 #include "utils.h"
 #include "version.h"
 
-void error_exit(char *format, ...)
+void error_exit(char *file, const char *function, int line, char *format, ...)
 {
 	va_list ap;
 
@@ -22,10 +27,13 @@ void error_exit(char *format, ...)
 	fprintf(stderr, version_str, VERSION);
 	fprintf(stderr, "\n\n");
 
+	fprintf(stderr,"A problem occured at line %d in function %s (from file %s):\n\n", line, function, file);
 	va_start(ap, format);
 	(void)vfprintf(stderr, format, ap);
 	va_end(ap);
-	if (errno) fprintf(stderr, "errno: %d=%s (if applicable)\n", errno, strerror(errno));
+	if (errno) fprintf(stderr, "\nerrno variable (if applicable): %d which means %s\n\n", errno, strerror(errno));
+
+	fprintf(stderr, "\nBinary build at %s %s\n", __DATE__, __TIME__);
 
 	dump_mem(SIGHUP);
 
