@@ -29,7 +29,7 @@ int start_tail(char *filename, char retry_open, char follow_filename, int initia
 	/* start child process */
 	if ((pid = fork()) == 0)
 	{
-		char *pars[9], *posix_version = NULL;
+		char *pars[16], *posix_version = NULL;
 		int npars = 0;
 		char nlines_buffer[32];
 
@@ -48,10 +48,6 @@ int start_tail(char *filename, char retry_open, char follow_filename, int initia
 		 */
 		if (retry_open)
 		{
-			/* FIXME: *//* #if defined(linux) */
-#if 0
-			pars[npars++] = "--retry";
-#else
 			int rc;
 			struct stat64 buf;
 
@@ -71,6 +67,9 @@ int start_tail(char *filename, char retry_open, char follow_filename, int initia
 
 				usleep(WAIT_FOR_FILE_DELAY * 1000);
 			}
+
+#if defined(linux) || defined(__CYGWIN__) || defined(__GNU__)
+			pars[npars++] = "--retry";
 #endif
 		}
 
@@ -96,6 +95,11 @@ int start_tail(char *filename, char retry_open, char follow_filename, int initia
 		else
 #endif
 		{
+#if !defined(linux) && !defined(__CYGWIN__) && !defined(__GNU__)
+			if (follow_filename && gnu_tail)
+				pars[npars++] = "--follow=name";
+#endif
+
 			/* check the posix compliance level */
 			if ((posix_version && atoi(posix_version) >= 200112) || posix_tail == MY_TRUE)
 			{
