@@ -3,11 +3,15 @@
 #include <sys/types.h>
 #include <regex.h>
 #include <ctype.h>
+#include <sys/socket.h>
+#include <netinet/in.h>
+
 #include "mt.h"
 #include "mem.h"
 #include "help.h"
 #include "term.h"
 #include "utils.h"
+#include "globals.h"
 
 char *help_main_menu[] = {
 		"MultiTail lets you create one or more windows in",
@@ -45,6 +49,8 @@ char *help_main_menu[] = {
 		"       easily see what has changed since the last",
 		"       time you looked",
 		" ^R^     restart a window",
+		" ^ctrl^ + ^v^  select conversion scheme(s)",
+		" ^ctrl^ + ^t^  toggle subwindow before lines",
 		" ^Y^     send a signal to a window (for commands)",
 		" ^y^     set linewrap",
 		" ^o^     wipe (clear) a window",
@@ -64,6 +70,7 @@ char *help_main_menu[] = {
 		"       with the current settings (the files that",
 		"       are monitored, regular expressions, etc.)",
 		" ^n^     clear the buffer of a window",
+		" ^N^     clear all buffers of all windows",
 		" ^m^     set the size of a buffer",
 		" ^k^     enter a (limited) terminal-mode",
 		" ^p^     pause all windows",
@@ -254,7 +261,7 @@ char *help_add_window_repeat_interval[] = {
 		"delay before restarting that command. The minimum",
 		"is 0: in that case the program gets repeated",
 		"immediately (that is: when MultiTail notices that",
-		"it exited!).",
+		"it died!).",
 		"Press enter on an empty line to abort.",
 		NULL
 	};
@@ -444,7 +451,7 @@ char *help_failed_to_start_tail[] = {
 char *help_window_closed[] = {
 		"A window has closed. This happens when you're",
 		"monitoring the output of an external program. That",
-		"program has exited and when that happens the",
+		"program has died and when that happens the",
 		"window is automatically closed.",
 		"On the commandline you can set with the ^-r^ or",
 		"the ^-R^ that a command should be repeated when it",
@@ -596,6 +603,10 @@ char *help_scrollback_help[] = {
 		" ^Y^     toggle linewrap. if linewrap is disabled,",
 		"       one can scroll left/right with the cursor-",
 		"       keys.",
+		" ^ctrl^+^t^ show (sub-)window number in front of",
+		"       each line",
+		" ^t^     show a list of window-nrs and what they",
+		"       contain. can also display statistics.",
 		" ^s^     save the buffer to a file",
 		"Press ^ctrl+g^ to abort.",
 		NULL
@@ -889,13 +900,55 @@ char *help_search_in_all_windows[] = {
 	};
 
 char *help_highlight_in_all_windows[] = {
-		/*2345678901234567890123456789012345678901234567890 */
 		"This functionality lets you search in all windows.",
 		"All lines with the search-string will be",
 		"highlighted (displayed in reverse).",
 		NULL
 	};
 
+char *help_select_conversionschemes[] = {
+		"With a conversionscheme one can let MultiTail",
+		"automatically convert e.g. ip-addresses, time-",
+		"stamps, etc. to something more readable. In this",
+		"menu you can select what conversionscheme(s) to",
+		"apply to a (sub-)window.",
+		NULL
+	};
+
+char *help_select_restart_window[] = {
+		"Restarting a window stops- and restarts the tail-",
+		"or command running underneath a (sub-)window.",
+		NULL
+	};
+
+char *help_select_restart_window_all[] = {
+		"This chooses wether to select all sub-windows or",
+		"just one specific.",
+		NULL
+	};
+
+char *help_select_terminal_emulation[] = {
+		"MultiTail can emulate a terminal. That way it can",
+		"display color (escape-)codes that sometimes are",
+		"in (log-)files.",
+		NULL
+	};
+
+char *help_clear_buffer[] = {
+		"Select window for which to clear the scrollback",
+		"buffer.",
+		NULL
+	};
+
+char *help_history[] = {
+		/*2345678901234567890123456789012345678901234567890 */
+		"Select a string from this list. These strings have",
+		"been used in the past in MultiTail. They are",
+		"stored in a file. Check the 'history_file' entry",
+		"in the configuration-file to see in what file.",
+		NULL
+	};
+
 #if 0
 char *help_bla[] = {
 		/*2345678901234567890123456789012345678901234567890 */
@@ -926,7 +979,6 @@ help_t	help[] =
 	{ HELP_ENTER_REGEXP_SELECT_SUBWINDOW,	help_enter_regexp_select_subwindow },
 	{ HELP_ENTER_STRIPPER_SELECT_SUBWINDOW,	help_enter_stripper_select_subwindow },
 	{ HELP_REGEXP_MENU,			help_regexp_menu },
-
 	{ HELP_ENTER_REGEXP,			help_enter_regexp },
 	{ HELP_NEGATE_REGEXP,			help_negate_regexp },
 	{ HELP_REGEXP_USAGE,			help_regexp_usage },
@@ -995,6 +1047,12 @@ help_t	help[] =
 	{ HELP_HIDE_BUT_WINDOW,			help_hide_but_window },
 	{ HELP_SEARCH_IN_ALL_WINDOWS,		help_search_in_all_windows },
 	{ HELP_HIGHLIGHT_IN_ALL_WINDOWS,	help_highlight_in_all_windows },
+	{ HELP_SELECT_CONVERSIONSCHEMES,	help_select_conversionschemes },
+	{ HELP_SELECT_RESTART_WINDOW,		help_select_restart_window },
+	{ HELP_SELECT_RESTART_WINDOW_ALL,	help_select_restart_window_all },
+	{ HELP_SELECT_TERMINAL_EMULATION,	help_select_terminal_emulation },
+	{ HELP_CLEAR_BUFFER,			help_clear_buffer },
+	{ HELP_HISTORY,				help_history },
 	{ -1, NULL }
 };
 
@@ -1072,10 +1130,14 @@ void show_help(int what_help)
 			{
 				pos = 0;
 			}
-			else if (tolower(c) == 'q' || c == 7)
+			else if (tolower(c) == 'q' || c == abort_key)
 			{
 				break;
 			}
+			else if (c == -1)
+			{
+				/* a window got closed */
+			}
 			else
 			{
 				wrong_key();
