@@ -13,6 +13,8 @@
 #include <time.h>
 #include <sys/time.h>
 #include <sys/resource.h>
+#include <sys/socket.h>
+#include <netinet/in.h>
 
 #include "mt.h"
 #include "term.h"
@@ -21,6 +23,7 @@
 #include "mem.h"
 #include "help.h"
 #include "globals.h"
+#include "ui.h"
 
 
 void info(void)
@@ -38,17 +41,19 @@ void info(void)
 		mvwprintw(mywin -> win, line++, 2, "Your terminal doesn't support colors");
 
 	if (uname(&uinfo) == -1)
-		error_exit("info: uname() failed\n");
-
-	line++;
-	mvwprintw(mywin -> win, line++, 2, "Running on:");
+		error_popup("Retrieving system information", -1, "uname() failed\n");
+	else
+	{
+		line++;
+		mvwprintw(mywin -> win, line++, 2, "Running on:");
 #ifdef _GNU_SOURCE
-	mvwprintw(mywin -> win, line++, 2, "%s/%s %s %s", uinfo.nodename, uinfo.sysname, uinfo.machine, uinfo.domainname);
+		mvwprintw(mywin -> win, line++, 2, "%s/%s %s %s", uinfo.nodename, uinfo.sysname, uinfo.machine, uinfo.domainname);
 #else
-	mvwprintw(mywin -> win, line++, 2, "%s/%s %s", uinfo.nodename, uinfo.sysname, uinfo.machine);
+		mvwprintw(mywin -> win, line++, 2, "%s/%s %s", uinfo.nodename, uinfo.sysname, uinfo.machine);
 #endif
-	mvwprintw(mywin -> win, line++, 2, "%s %s", uinfo.release, uinfo.version);
-	line++;
+		mvwprintw(mywin -> win, line++, 2, "%s %s", uinfo.release, uinfo.version);
+		line++;
+	}
 
 	if (has_colors())
 		mvwprintw(mywin -> win, line++, 2, "colors: %d, colorpairs: %d (%d), change colors: %s", COLORS, COLOR_PAIRS, cp.n_def, can_change_color()?"yes":"no");
@@ -66,26 +71,29 @@ void info(void)
 
 	escape_print(mywin, 16, 2, "_Press any key to exit this screen_");
 
-#if defined(__FreeBSD__) || defined(linux) || defined(__OpenBSD__) || defined(__NetBSD__) || defined(__APPLE__) || defined(sun) || defined(__sun) || defined(__GNU__)
+#if defined(__FreeBSD__) || defined(linux) || defined(__OpenBSD__) || defined(__NetBSD__) || defined(__APPLE__) || defined(sun) || defined(__sun) || defined(__GNU__) || defined(__CYGWIN__)
 	for(;;)
 	{
-		int run_time = time(NULL) - mt_started;
+		dtime_t run_time = get_ts() - mt_started;
+#ifndef __CYGWIN__
 		double v1, v2, v3;
 
 		get_load_values(&v1, &v2, &v3);
 		mvwprintw(mywin -> win, 6, 2, "Current load of system: %f %f %f", v1, v2, v3);
+#endif
 
 		if (run_time)
 		{
 			struct rusage usage;
 
 			if (getrusage(RUSAGE_SELF, &usage) == -1)
-				error_exit("info: getrusage() failed\n");
+				error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "getrusage() failed\n");
 
-			mvwprintw(mywin -> win, proc_u_line, 2, "Runtime: %02d:%02d:%02d, average processor usage: %.2f%%  ",
-					(run_time / 3600), (run_time / 60) % 60, run_time % 60,
+			mvwprintw(mywin -> win, proc_u_line, 2, "Runtime: %02d:%02d:%02d, avg.proc.usage: %.2f%% ints/s: %.1f",
+					(int)(run_time / 3600), ((int)run_time / 60) % 60, (int)run_time % 60,
 					((double)usage.ru_utime.tv_sec + (double)usage.ru_utime.tv_usec / 1000000.0 + 
-					 (double)usage.ru_stime.tv_sec + (double)usage.ru_stime.tv_usec / 1000000.0) * 100.0 / (double)run_time);
+					 (double)usage.ru_stime.tv_sec + (double)usage.ru_stime.tv_usec / 1000000.0) * 100.0 / run_time,
+					 (double)total_wakeups / run_time);
 		}
 
 		mydoupdate();
@@ -119,9 +127,11 @@ void statistics_popup(int f_index, proginfo *cur)
 
 	for(;;)
 	{
-		time_t time_running = time(NULL) - cur -> statistics.start_ts;
-		char *start_ts_str = mystrdup(ctime(&cur -> statistics.start_ts), "start ts");
-		char *last_ts_str  = mystrdup(ctime(&cur -> statistics.lastevent), "last ts");
+		dtime_t time_running = get_ts() - cur -> statistics.start_ts;
+		time_t start_ts = (time_t)cur -> statistics.start_ts;
+		char *start_ts_str = mystrdup(ctime(&start_ts),  __FILE__, __PRETTY_FUNCTION__, __LINE__);
+		time_t lastevent = (time_t)cur -> statistics.lastevent;
+		char *last_ts_str  = mystrdup(ctime(&lastevent), __FILE__, __PRETTY_FUNCTION__, __LINE__);
 		char *dummy;
 		char *vmsize_str = NULL;
 		char *fsize_str = NULL;
@@ -132,11 +142,11 @@ void statistics_popup(int f_index, proginfo *cur)
 		int total_re = 0;
 		int loop;
 
-		if (cur -> is_command)
+		if (cur -> wt == WT_COMMAND)
 		{
 			vmsize_str = amount_to_str(get_vmsize(cur -> pid));
 		}
-		else if (!cur -> is_stdin)
+		else if (cur -> wt == WT_FILE)
 		{
 			(void)file_info(cur -> filename, &fsize, 0, NULL, NULL);
 			fsize_str = amount_to_str(fsize);
@@ -155,9 +165,9 @@ void statistics_popup(int f_index, proginfo *cur)
 		mvwprintw(popup -> win, 3, 27, "#l/s :");
 		mvwprintw(popup -> win, 3, 44, "Avg len:");
 		mvwprintw(popup -> win, 4, 2, "Data interval :");
-		if (cur -> is_command)
+		if (cur -> wt == WT_COMMAND)
 			mvwprintw(popup -> win, 5, 2, "VM size       :");
-		else
+		else if (cur -> wt == WT_FILE)
 			mvwprintw(popup -> win, 5, 2, "File size     :");
 		mvwprintw(popup -> win, 9, 2, "Data processed:");
 		mvwprintw(popup -> win, 9, 27, "Bps  :");
@@ -169,7 +179,7 @@ void statistics_popup(int f_index, proginfo *cur)
 		mvwprintw(popup -> win, 11, 27, "Bytes:");
 		mvwprintw(popup -> win, 11, 44, "Limit  :");
 		mvwprintw(popup -> win, 12, 2, "# of beeps:    ");
-		if (cur -> is_command)
+		if (cur -> wt == WT_COMMAND)
 		{
 			mvwprintw(popup -> win, 13, 2, "Number of runs:");
 			mvwprintw(popup -> win, 13, 27, "Last rc:");
@@ -178,7 +188,7 @@ void statistics_popup(int f_index, proginfo *cur)
 
 		mvwprintw(popup -> win, 3, 18, "%d", cur -> statistics.n_events);
 		mvwprintw(popup -> win, 6, 18, "%s", start_ts_str);
-		if (cur -> statistics.lastevent != (time_t)0)
+		if (cur -> statistics.lastevent != (dtime_t)0.0)
 			mvwprintw(popup -> win, 7, 18, "%s", last_ts_str);
 		else
 			mvwprintw(popup -> win, 7, 18, "---");
@@ -208,7 +218,7 @@ void statistics_popup(int f_index, proginfo *cur)
 			{
 				double dummy_d = (double)(time(NULL) - cur -> statistics.lastevent) / avg;
 				time_t next_event = cur -> statistics.lastevent + (ceil(dummy_d) * avg);
-				char *ne_str = mystrdup(ctime(&next_event), "next event time string");
+				char *ne_str = mystrdup(ctime(&next_event), __FILE__, __PRETTY_FUNCTION__, __LINE__);
 				char *dummy_str = strchr(ne_str, '\n');
 
 				if (dummy_str) *dummy_str = 0x00;
@@ -218,11 +228,11 @@ void statistics_popup(int f_index, proginfo *cur)
 
 			mvwprintw(popup -> win, 3, 53, "%.1f", (double)cur -> statistics.bytes_processed / (double)cur -> statistics.n_events);
 		}
-		if (cur -> is_command)
+		if (cur -> wt == WT_COMMAND)
 			mvwprintw(popup -> win, 5, 18, "%s", vmsize_str);
-		else if (cur -> is_stdin)
+		else if (cur -> wt == WT_STDIN || cur -> wt == WT_SOCKET)
 			mvwprintw(popup -> win, 5, 18, "n.a.");
-		else
+		else if (cur -> wt == WT_FILE)
 			mvwprintw(popup -> win, 5, 18, "%s", fsize_str);
 		myfree(vmsize_str);
 		myfree(fsize_str);
@@ -250,7 +260,7 @@ void statistics_popup(int f_index, proginfo *cur)
 			mvwprintw(popup -> win, 10, 18, "%d (%.2f%%)", total_re, (total_re * 100.0) / (double)cur -> statistics.n_events);
 		else
 			mvwprintw(popup -> win, 10, 18, "%d", total_re);
-		if (cur -> is_command)
+		if (cur -> wt == WT_COMMAND)
 		{
 			mvwprintw(popup -> win, 13, 18, "%d", cur -> n_runs);
 			mvwprintw(popup -> win, 13, 36, "%d", cur -> last_exit_rc);
@@ -265,7 +275,7 @@ void statistics_popup(int f_index, proginfo *cur)
 			mvwprintw(popup -> win, 11, 53, "%s", str);
 			myfree(str);
 		}
-		wborder(popup -> win, box_left_side, box_right_side, box_top_side, box_bottom_side, box_top_left_hand_corner, box_top_right_hand_corner, box_bottom_left_hand_corner, box_bottom_right_hand_corner);
+		draw_border(popup);
 		mydoupdate();
 
 		c = toupper(wait_for_keypress(HELP_STATISTICS_POPUP, popup_refresh_interval, popup, 0));
@@ -290,38 +300,38 @@ void statistics_popup(int f_index, proginfo *cur)
 void statistics_menu(void)
 {
 	NEWWIN *mywin = create_popup(23, 65);
-	proginfo **plist = NULL;
-	char     *issub = NULL;
-	int      *winnr = NULL;
-	int loop, nwin = 0;
 	int offset = 0, cur_line = 0;
 
-	/* create list of (sub-)windows */
-	for(loop=0; loop<nfd; loop++)
-	{
-		proginfo *cur = &pi[loop];
-
-		while(cur)
-		{
-			plist = (proginfo **)myrealloc(plist, (nwin + 1) * sizeof(proginfo *), "(sub-)windows pointer list");
-			issub = (char *)     myrealloc(issub, (nwin + 1) * sizeof(char)       , "(sub-)windows list subwindow indicator");
-			winnr = (int *)      myrealloc(winnr, (nwin + 1) * sizeof(int)        , "(sub-)windows list windor nr");
-
-			plist[nwin] = cur;
-			issub[nwin] = (cur != &pi[loop]);
-			winnr[nwin] = loop;
-			nwin++;
-
-			cur = cur -> next;
-		}
-	}
-
 	for(;;)
 	{
 		int c;
 		int vmsize = get_vmsize(getpid());
 		time_t now = time(NULL);
 		struct tm *tmnow = localtime(&now);
+		proginfo **plist = NULL;
+		char     *issub = NULL;
+		int      *winnr = NULL;
+		int loop, nwin = 0;
+
+		/* create list of (sub-)windows */
+		for(loop=0; loop<nfd; loop++)
+		{
+			proginfo *cur = &pi[loop];
+
+			while(cur)
+			{
+				plist = (proginfo **)myrealloc(plist, (nwin + 1) * sizeof(proginfo *), __FILE__, __PRETTY_FUNCTION__, __LINE__);
+				issub = (char *)     myrealloc(issub, (nwin + 1) * sizeof(char)      , __FILE__, __PRETTY_FUNCTION__, __LINE__);
+				winnr = (int *)      myrealloc(winnr, (nwin + 1) * sizeof(int)       , __FILE__, __PRETTY_FUNCTION__, __LINE__);
+
+				plist[nwin] = cur;
+				issub[nwin] = (cur != &pi[loop]);
+				winnr[nwin] = loop;
+				nwin++;
+
+				cur = cur -> next;
+			}
+		}
 
 		werase(mywin -> win);
 		win_header(mywin, "Statistics");
@@ -343,7 +353,7 @@ void statistics_menu(void)
 			if (loop == cur_line) ui_inverse_off(mywin);
 		}
 
-		mvwprintw(mywin -> win, 20, 2, "Run-time: %.2f hours  %02d:%02d", difftime(time(NULL), mt_started) / 3600.0, tmnow -> tm_hour, tmnow -> tm_min);
+		mvwprintw(mywin -> win, 20, 2, "Run-time: %.2f hours  %02d:%02d", (get_ts() - mt_started) / 3600.0, tmnow -> tm_hour, tmnow -> tm_min);
 		if (vmsize != -1)
 		{
 			char *vmsize_str = amount_to_str(vmsize);
@@ -352,7 +362,7 @@ void statistics_menu(void)
 		}
 		escape_print(mywin, 21, 2, "Press ^r^ to reset counters, ^q^ to exit");
 
-		wborder(mywin -> win, box_left_side, box_right_side, box_top_side, box_bottom_side, box_top_left_hand_corner, box_top_right_hand_corner, box_bottom_left_hand_corner, box_bottom_right_hand_corner);
+		draw_border(mywin);
 		mydoupdate();
 
 		c = toupper(wait_for_keypress(HELP_STATISTICS, popup_refresh_interval, mywin, 1));
@@ -399,18 +409,21 @@ void statistics_menu(void)
 		}
 		else if (c == 'Q' || c == abort_key)
 		{
+			myfree(plist);
+			myfree(issub);
+			myfree(winnr);
 			break;
 		}
 		else if (c != -1)
 		{
 			wrong_key();
 		}
+
+		myfree(plist);
+		myfree(issub);
+		myfree(winnr);
 	}
 
-
-	myfree(plist);
-	myfree(issub);
-
 	delete_popup(mywin);
 }
 
@@ -464,15 +477,18 @@ void heartbeat(void)
 	mydoupdate();
 }
 
-void do_check_for_mail(time_t ts)
+void do_check_for_mail(dtime_t now)
 {
-	if (check_for_mail > 0 && mail_spool_file != NULL && difftime(ts, msf_last_check) >= check_for_mail)
+	if (check_for_mail > 0 && mail_spool_file != NULL && (now - msf_last_check) >= check_for_mail)
 	{
 		/* get current filesize */
 		if (stat64(mail_spool_file, &msf_info) == -1)
 		{
-			if (errno != ENOENT)
-				error_exit("do_check_for_mail: Error doing stat64() on file %s\n", mail_spool_file);
+			if (errno != ENOENT) 
+			{
+				check_for_mail = 0;
+				error_popup("Check for new e-mail", -1, "Error doing stat64() on file %s.\ne-Mail check disabled.\n", mail_spool_file);
+			}
 		}
 
 		/* filesize changed? */
@@ -494,15 +510,15 @@ void do_check_for_mail(time_t ts)
 			msf_prev_size = msf_info.st_size;
 		}
 
-		msf_last_check = ts;
+		msf_last_check = now;
 	}
 }
 
-void store_statistics(proginfo *cur, time_t now)
+void store_statistics(proginfo *cur, dtime_t now)
 {
 	if (cur -> statistics.lastevent)
 	{
-		double cur_deltat = difftime(now, cur -> statistics.lastevent);
+		dtime_t cur_deltat = now - cur -> statistics.lastevent;
 
 		if (cur -> statistics.n_events == 1)
 		{
