@@ -1,22 +1,30 @@
-#define WAIT_FOR_FILE_DELAY	250	/* sleep when waiting for a file to become available */
-#define MAX_N_RE_MATCHES	80	/* max. number of regex matches: used for coloring matches */
-#define MIN_N_BUFFERLINES	100	/* number of lines to buffer at minimum */
-#define MAX_N_SPAWNED_PROCESSES	16	/* max. nr. of processes executed by matching regexps */
-#define MAX_N_COLUMNS		15	/* max number of columns */
-#define DEFAULT_TAB_WIDTH	4
-#define DEFAULT_COLORPAIRS	8
-#define MAX_BACKTRACE_LENGTH	256
-#define SCRIPT_IO_BUFFER_SIZE	4096
-#define CONFIG_READ_BUFFER	4096
-#define HISTORY_IO_BUFFER	4096
+#define WAIT_FOR_FILE_DELAY	(250)	/* sleep when waiting for a file to become available */
+#define MAX_N_RE_MATCHES	(80)	/* max. number of regex matches: used for coloring matches */
+#define MIN_N_BUFFERLINES	(100)	/* number of lines to buffer at minimum */
+#define MAX_N_SPAWNED_PROCESSES	(16)	/* max. nr. of processes executed by matching regexps */
+#define MAX_N_COLUMNS		(15)	/* max number of columns */
+#define DEFAULT_TAB_WIDTH	(4)
+#define DEFAULT_COLORPAIRS	(8)
+#define MAX_BACKTRACE_LENGTH	(256)
+#define SCRIPT_IO_BUFFER_SIZE	(4096)
+#define CONFIG_READ_BUFFER	(4096)
+#define HISTORY_IO_BUFFER	(4096)
+#define TIMESTAMP_EXTEND_BUFFER	(1024)
 
-#define SL_REGULAR	1
-#define SL_NONE		0
-#define SL_ATTR		2
+#define SL_REGULAR	(1)
+#define SL_NONE		(0)
+#define SL_ATTR		(2)
 
-#define LOADAVG_STR_LEN		20
+#define LOADAVG_STR_LEN		(20)
 #define AMOUNT_STR_LEN		(3 + 2 + 1)
 
+#ifndef __GNUC__
+        #define __PRETTY_FUNCTION__	"(unknown)"
+	#define USE_IF_SET(x, y)		((x)?(x):(y))
+#else
+	#define USE_IF_SET(x, y)		((x)?:(y))
+#endif
+
 typedef enum { MY_FALSE = 0, MY_TRUE = 1 } mybool_t;
 
 typedef enum { VAL_ZERO_POSITIVE = 1, VAL_POSITIVE_NOT_1, VAL_POSITIVE } valcheck_t;
@@ -27,12 +35,12 @@ typedef enum { VAL_ZERO_POSITIVE = 1, VAL_POSITIVE_NOT_1, VAL_POSITIVE } valchec
 
 typedef enum { BEEP_FLASH = 1, BEEP_BEEP, BEEP_POPUP, BEEP_NONE } beeb_t;
 
-typedef enum { REDIRECTTO_NONE = 0, REDIRECTTO_PIPE_FILTERED = 1, REDIRECTTO_PIPE, REDIRECTTO_FILE_FILTERED, REDIRECTTO_FILE } redirecttype_t;
-
 typedef enum { LINE_LEFT = 1, LINE_RIGHT, LINE_TOP, LINE_BOTTOM } linepos_t;
 
 typedef enum { TERM_IGNORE = 0, TERM_XTERM, TERM_ANSI /* or vt100 */} term_t;
 
+typedef enum { SCHEME_TYPE_EDIT = 0, SCHEME_TYPE_FILTER } filter_edit_scheme_t;
+
 #ifndef _BSD_SOURCE
 #define _BSD_SOURCE	/* don't worry: it'll still work if you don't have a BSD system */
 #endif
@@ -76,16 +84,18 @@ typedef enum { TERM_IGNORE = 0, TERM_XTERM, TERM_ANSI /* or vt100 */} term_t;
 #define open64 open
 #endif
 
-#define MARKER_REGULAR	NULL
-#define MARKER_CHANGE	(proginfo *)-1
-#define MARKER_IDLE	(proginfo *)-2
-#define MARKER_MSG	(proginfo *)-3
+#define MARKER_REGULAR	(NULL)
+#define MARKER_CHANGE	((proginfo *)-1)
+#define MARKER_IDLE	((proginfo *)-2)
+#define MARKER_MSG	((proginfo *)-3)
 #define IS_MARKERLINE(x)	((x) == MARKER_REGULAR || (x) == MARKER_CHANGE || (x) == MARKER_IDLE || (x) == MARKER_MSG)
 
 typedef enum { SEL_WIN = 1, SEL_SUBWIN, SEL_FILES, SEL_CSCHEME, SEL_HISTORY } selbox_type_t;
 
 typedef enum { TT_ATIME = 1, TT_MTIME, TT_CTIME } time_field_t;
 
+typedef double dtime_t;
+
 typedef struct
 {
 	char *history_file;
@@ -105,7 +115,7 @@ typedef struct
 {
 	char *glob_str;
 	int check_interval;
-	time_t last_check;
+	dtime_t last_check;
 	time_field_t new_only;
 
 	char in_one_window;
@@ -143,7 +153,7 @@ typedef struct
 	re *pre;
 } filterscheme;
 
-typedef enum { STRIP_TYPE_REGEXP = 1, STRIP_TYPE_RANGE, STRIP_TYPE_COLUMN } striptype_t;
+typedef enum { STRIP_TYPE_REGEXP = 1, STRIP_TYPE_RANGE, STRIP_TYPE_COLUMN, STRIP_KEEP_SUBSTR } striptype_t;
 
 typedef struct
 {
@@ -169,7 +179,7 @@ typedef struct
 	strip_t *strips;
 } editscheme;
 
-#define MAX_COLORS_PER_LINE 80
+#define MAX_COLORS_PER_LINE (80)
 
 typedef struct
 {
@@ -188,16 +198,20 @@ typedef struct
 	int attrs;
 } myattr_t;
 
+typedef enum { REDIRECTTO_NONE = 0, REDIRECTTO_PIPE_FILTERED = 1, REDIRECTTO_PIPE, REDIRECTTO_FILE_FILTERED, REDIRECTTO_FILE, REDIRECTTO_SOCKET_FILTERED, REDIRECTTO_SOCKET } redirecttype_t;
+
 typedef struct {
 	char *redirect;
 	redirecttype_t type;
 	int fd;
 	pid_t pid;
+	struct sockaddr_in sai;
+	int prio_fac;
 } redirect_t;
 
 typedef struct
 {	
-	time_t lastevent;
+	dtime_t lastevent;
 	double prev_deltat, total_deltat;
 	double med;
 	double dev;
@@ -205,7 +219,7 @@ typedef struct
 	double scc, sccu0, scclast, scct1, scct2, scct3;
 	int n_events;
 
-	time_t start_ts;
+	dtime_t start_ts;
 	long long int bytes_processed;
 } statistics_t;
 
@@ -246,13 +260,14 @@ typedef struct
 	int n_times_repeated;
 } repeatinglines_t;
 
+typedef enum { WT_FILE=1, WT_COMMAND, WT_STDIN, WT_SOCKET } windowtype_t;
+
 typedef struct _subwindow_
 {
 	char *filename;
-	char is_command;
+	windowtype_t wt;
 	int last_exit_rc;
 	int n_runs;
-	char is_stdin;
 	int check_interval;
 	int fd;		/* read */
 	int wfd;	/* write */
@@ -385,7 +400,7 @@ typedef struct
 	color_scheme_entry *pentries;
 } color_scheme;
 
-typedef enum { CONVTYPE_IP4TOHOST = 1, CONVTYPE_EPOCHTODATE, CONVTYPE_ERRNO, CONVTYPE_HEXTODEC, CONVTYPE_DECTOHEX, CONVTYPE_TAI64NTODATE, CONVTYPE_SCRIPT, CONVTYPE_ABBRTOK } conversion_t;
+typedef enum { CONVTYPE_IP4TOHOST = 1, CONVTYPE_EPOCHTODATE, CONVTYPE_ERRNO, CONVTYPE_HEXTODEC, CONVTYPE_DECTOHEX, CONVTYPE_TAI64NTODATE, CONVTYPE_SCRIPT, CONVTYPE_ABBRTOK, CONVTYPE_SIGNRTOSTRING } conversion_t;
 typedef struct
 {
 	conversion_t type;
