@@ -103,17 +103,20 @@ void scrollback_savefile(buffer *pbuf)
 			{
 				if ((pbuf -> be)[loop].Bline)
 				{
-					char display;
+					char display = 1;
 					char *error;
 					int dummy = -1;
 					regmatch_t *pmatch = NULL;
 
 					/* check filter */
-					(void)check_filter((pbuf -> be)[loop].pi, (pbuf -> be)[loop].Bline, &pmatch, &error, &dummy, 0, &display);
-					if (error)
+					if (!IS_MARKERLINE((pbuf -> be)[loop].pi))
 					{
-						fprintf(fh, "%s\n", error);
-						myfree(error);
+						(void)check_filter((pbuf -> be)[loop].pi, (pbuf -> be)[loop].Bline, &pmatch, &error, &dummy, 0, &display);
+						if (error)
+						{
+							fprintf(fh, "%s\n", error);
+							myfree(error);
+						}
 					}
 					if (display)
 					{
@@ -219,6 +222,7 @@ int scrollback_do(int window_nr, buffer *pbuf, int *winnrs, char *header)
 	char show_winnr = default_sb_showwinnr;
 	mybool_t case_insensitive = re_case_insensitive;
 
+	LOG("---\n");
 	if (global_highlight_str)
 	{
 		find = mystrdup(global_highlight_str, __FILE__, __PRETTY_FUNCTION__, __LINE__);
@@ -516,7 +520,7 @@ int scrollback_do(int window_nr, buffer *pbuf, int *winnrs, char *header)
 					}
 				}
 			}
-					
+
 			redraw = 2;
 		}
 		else if ((c == KEY_HOME ||
@@ -857,6 +861,7 @@ int scrollback_search_to_new_window(buffer *pbuf, char *org_title, char *find_st
 			cur_lb.be = myrealloc(cur_lb.be, (cur_lb.curpos + 1) * sizeof(buffered_entry), __FILE__, __PRETTY_FUNCTION__, __LINE__);
 			cur_lb.be[cur_lb.curpos].Bline = (pbuf -> be)[loop].Bline;
 			cur_lb.be[cur_lb.curpos].pi    = (pbuf -> be)[loop].pi;
+			cur_lb.be[cur_lb.curpos].ts    = (pbuf -> be)[loop].ts;
 			cur_lb.curpos++;
 		}
 	}
