@@ -5,6 +5,8 @@
 #include <string.h>
 #include <sys/stat.h>
 #include <unistd.h>
+#include <sys/socket.h>
+#include <netinet/in.h>
 
 #include "mt.h"
 #include "term.h"
@@ -30,16 +32,16 @@ int create_subwindow_list(int f_index, char ***swlist)
 
 	do
 	{
-		list = (char **)myrealloc(list, (n + 1) * sizeof(char *), "create_subwindow_list: subwindow list");
+		list = (char **)myrealloc(list, (n + 1) * sizeof(char *), __FILE__, __PRETTY_FUNCTION__, __LINE__);
 
 		if (show_subwindow_id)
 		{
 			int len = strlen(cur -> filename);
-			list[n] = (char *)mymalloc(len + 1, "create_subwindow_list: subwindow filename");
+			list[n] = (char *)mymalloc(len + 1, __FILE__, __PRETTY_FUNCTION__, __LINE__);
 			memcpy(list[n], cur -> filename, len + 1);
 		}
 		else
-			list[n] = mystrdup(cur -> filename, "create_subwindow_list: subwindowlist filename");
+			list[n] = mystrdup(cur -> filename, __FILE__, __PRETTY_FUNCTION__, __LINE__);
 
 		n++;
 
@@ -103,9 +105,9 @@ int selection_box(void **list, char *needs_mark, int nlines, selbox_type_t type,
 	int pos = 0, ppos = -1, offs = 0, poffs = -1;
 	int loop = 0, sel = -1;
 	char first = 1;
-	char *dummy = (char *)mymalloc(wcols + 1, "selection_box: windowname string");
+	char *dummy = (char *)mymalloc(wcols + 1, __FILE__, __PRETTY_FUNCTION__, __LINE__);
 	int path_max = find_path_max();
-	char *selstr = (char *)mymalloc(path_max + 1, "selection_box: enter selection"), selfound = 0;
+	char *selstr = (char *)mymalloc(path_max + 1, __FILE__, __PRETTY_FUNCTION__, __LINE__), selfound = 0;
 
 	selstr[0] = 0x00;
 
@@ -149,7 +151,7 @@ int selection_box(void **list, char *needs_mark, int nlines, selbox_type_t type,
 					ui_inverse_off(mywin);
 			}
 
-			wborder(mywin -> win, box_left_side, box_right_side, box_top_side, box_bottom_side, box_top_left_hand_corner, box_top_right_hand_corner, box_bottom_left_hand_corner, box_bottom_right_hand_corner);
+			draw_border(mywin);
 
 			ppos = pos;
 			poffs = offs;
@@ -269,7 +271,7 @@ int selection_box(void **list, char *needs_mark, int nlines, selbox_type_t type,
 			sel = pos + offs;
 			break;
 		}
-		else if (c == abort_key)
+		else if (c == abort_key || c == -1)
 		{
 			break;
 		}
@@ -362,6 +364,7 @@ char * select_file(char *input, int what_help)
 	char *new_fname = NULL;
 	struct stat64 statbuf;
 	int list_n, index;
+	int strbufsize = find_path_max();
 
 	list_n = match_files(input, &path, &list, &isdir);
 	if (list_n == 0)
@@ -374,9 +377,9 @@ char * select_file(char *input, int what_help)
 	index = selection_box((void **)list, isdir, list_n, SEL_FILES, what_help, NULL);
 	if (index != -1)
 	{
-		new_fname = (char *)mymalloc(find_path_max() + 1, "select_file: new filename");
+		new_fname = (char *)mymalloc(strbufsize + 1, __FILE__, __PRETTY_FUNCTION__, __LINE__);
 
-		snprintf(new_fname, find_path_max(), "%s%s", path, list[index]);
+		snprintf(new_fname, strbufsize, "%s%s", path, list[index]);
 
 		if (stat64(new_fname, &statbuf) == -1)
 		{
@@ -388,7 +391,7 @@ char * select_file(char *input, int what_help)
 		{
 			if (S_ISDIR(statbuf.st_mode))
 			{
-				strcat(new_fname, "/");
+				strncat(new_fname, "/", strbufsize);
 			}
 		}
 	}
