@@ -1594,7 +1594,7 @@ void write_script(void)
 			{
 				int lloop;
 
-				if (cur -> beep.beep_interval != -1)
+				if (cur -> beep.beep_interval > 0)
 					fprintf(fh, " --bi %d", cur -> beep.beep_interval);
 
 				if (cur -> check_interval)
@@ -2424,6 +2424,29 @@ void set_linewrap(void)
 	delete_popup(mywin);
 }
 
+void horizontal_scroll(int direction)
+{
+	proginfo *cur = &pi[0];
+
+	do
+	{
+		if (cur->line_wrap == 'l')
+		{
+			if (direction == KEY_RIGHT)
+			{
+				cur->line_wrap_offset++;
+			}
+			else
+			{
+				if (cur->line_wrap_offset > 0)
+					cur->line_wrap_offset--;
+			}
+		}
+
+		cur= cur->next;
+	} while(cur);
+}
+
 void regexp_error_popup(int rc, regex_t *pre)
 {
 	char popup_buffer[4096];
